@@ -25,10 +25,11 @@
  * @author Christian Grothoff
  */
 
+#include "platform.h"
 #include "fs.h"
 #include "search.h"
+#include "status.h"
 #include "meta.h"
-#include "platform.h"
 
 /* ****************** FSUI download events ****************** */
 
@@ -39,8 +40,8 @@
  * by the download list.
  */
 static int
-addFilesToDirectory (const ECRS_FileInfo * fi,
-                     const HashCode512 * key, int isRoot, void *closure)
+addFilesToDirectory (const GNUNET_ECRS_FileInfo * fi,
+                     const GNUNET_HashCode * key, int isRoot, void *closure)
 {
   DownloadList *list = closure;
   GtkTreeIter iter;
@@ -49,16 +50,16 @@ addFilesToDirectory (const ECRS_FileInfo * fi,
   GtkTreePath *path;
   GtkTreeModel *model;
 
-  if (isRoot == YES)
-    return OK;
+  if (isRoot == GNUNET_YES)
+    return GNUNET_OK;
   if (!gtk_tree_row_reference_valid (list->searchViewRowReference))
-    return SYSERR;
+    return GNUNET_SYSERR;
   model = GTK_TREE_MODEL (list->searchList->tree);
   path = gtk_tree_row_reference_get_path (list->searchViewRowReference);
   if (path == NULL)
     {
-      GE_BREAK (ectx, 0);
-      return SYSERR;
+      GNUNET_GE_BREAK (ectx, 0);
+      return GNUNET_SYSERR;
     }
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_path_free (path);
@@ -68,30 +69,30 @@ addFilesToDirectory (const ECRS_FileInfo * fi,
     {
       if (TRUE == gtk_tree_model_iter_nth_child (model, &child, &iter, i))
         {
-          struct ECRS_URI *uri;
+          struct GNUNET_ECRS_URI *uri;
           uri = NULL;
           gtk_tree_model_get (model, &child, SEARCH_URI, &uri, -1);
-          if ((uri != NULL) && (ECRS_equalsUri (uri, fi->uri)))
-            return OK;
+          if ((uri != NULL) && (GNUNET_ECRS_uri_test_equal (uri, fi->uri)))
+            return GNUNET_OK;
         }
     }
   gtk_tree_store_append (GTK_TREE_STORE (model), &child, &iter);
   addEntryToSearchTree (list->searchList, list, fi, &child);
-  return OK;
+  return GNUNET_OK;
 }
 
 static void
 refreshDirectoryViewFromDisk (DownloadList * list)
 {
   unsigned long long size;
-  char *data;
+  const char *data;
   int fd;
   char *fn;
-  struct ECRS_MetaData *meta;
+  struct GNUNET_MetaData *meta;
   struct stat buf;
   const char *f;
 
-  if ((list->is_directory != YES) ||
+  if ((list->is_directory != GNUNET_YES) ||
       (list->searchList == NULL) ||
       (list->searchViewRowReference == NULL) ||
       (!gtk_tree_row_reference_valid (list->searchViewRowReference)))
@@ -102,14 +103,15 @@ refreshDirectoryViewFromDisk (DownloadList * list)
   if (S_ISDIR (buf.st_mode))
     {
       fn =
-        MALLOC (strlen (list->filename) + strlen (GNUNET_DIRECTORY_EXT) + 1);
+        GNUNET_malloc (strlen (list->filename) +
+                       strlen (GNUNET_DIRECTORY_EXT) + 1);
       strcpy (fn, list->filename);
       if (fn[strlen (fn) - 1] == '/')
         fn[strlen (fn) - 1] = '\0';
       strcat (fn, GNUNET_DIRECTORY_EXT);
       if (0 != stat (list->filename, &buf))
         {
-          FREE (fn);
+          GNUNET_free (fn);
           return;
         }
       f = fn;
@@ -122,30 +124,33 @@ refreshDirectoryViewFromDisk (DownloadList * list)
   size = buf.st_size;
   if (size == 0)
     {
-      FREENONNULL (fn);
+      GNUNET_free_non_null (fn);
       return;
     }
-  fd = disk_file_open (ectx, list->filename, O_RDONLY);
+  fd = GNUNET_disk_file_open (ectx, f, O_RDONLY);
   if (fd == -1)
     {
-      FREENONNULL (fn);
+      GNUNET_free_non_null (fn);
       return;
     }
   data = MMAP (NULL, size, PROT_READ, MAP_SHARED, fd, 0);
   if ((data == MAP_FAILED) || (data == NULL))
     {
-      GE_LOG_STRERROR_FILE (ectx, GE_ERROR | GE_ADMIN | GE_BULK, "mmap", f);
+      GNUNET_GE_LOG_STRERROR_FILE (ectx,
+                                   GNUNET_GE_ERROR | GNUNET_GE_ADMIN |
+                                   GNUNET_GE_BULK, "mmap", f);
       CLOSE (fd);
-      FREENONNULL (fn);
+      GNUNET_free_non_null (fn);
       return;
     }
-  FREENONNULL (fn);
+  GNUNET_free_non_null (fn);
   meta = NULL;
-  ECRS_listDirectory (ectx, data, size, &meta, &addFilesToDirectory, list);
-  MUNMAP (data, size);
+  GNUNET_ECRS_directory_list_contents (ectx, data, size, NULL, &meta,
+                                       &addFilesToDirectory, list);
+  MUNMAP ((void *) data, size);
   CLOSE (fd);
   if (meta != NULL)
-    ECRS_freeMetaData (meta);
+    GNUNET_meta_data_destroy (meta);
 }
 
 /**
@@ -154,15 +159,15 @@ refreshDirectoryViewFromDisk (DownloadList * list)
  * the download summary.
  */
 DownloadList *
-fs_download_started (struct FSUI_DownloadList *fsui_dl,
+fs_download_started (struct GNUNET_FSUI_DownloadList *fsui_dl,
                      DownloadList * dl_parent,
                      SearchList * sl_parent,
                      unsigned long long total,
                      unsigned int anonymityLevel,
-                     const ECRS_FileInfo * fi,
+                     const GNUNET_ECRS_FileInfo * fi,
                      const char *filename,
                      unsigned long long completed,
-                     cron_t eta, FSUI_State state)
+                     GNUNET_CronTime eta, GNUNET_FSUI_State state)
 {
   DownloadList *list;
   GtkTreeIter iter;
@@ -174,19 +179,19 @@ fs_download_started (struct FSUI_DownloadList *fsui_dl,
   int progress;
   char *uri_name;
   gboolean valid;
-  struct ECRS_URI *u;
+  struct GNUNET_ECRS_URI *u;
   GtkTreeModel *model;
 
   /* setup visualization */
-  list = MALLOC (sizeof (DownloadList));
+  list = GNUNET_malloc (sizeof (DownloadList));
   memset (list, 0, sizeof (DownloadList));
-  list->uri = ECRS_dupUri (fi->uri);
-  list->filename = STRDUP (filename);
+  list->uri = GNUNET_ECRS_uri_duplicate (fi->uri);
+  list->filename = GNUNET_strdup (filename);
   if ((dl_parent != NULL) &&
       (NULL !=
        (path =
-        gtk_tree_row_reference_get_path (dl_parent->
-                                         summaryViewRowReference))))
+        gtk_tree_row_reference_get_path
+        (dl_parent->summaryViewRowReference))))
     {
       valid = gtk_tree_model_get_iter (GTK_TREE_MODEL (download_summary),
                                        &piter, path);
@@ -204,8 +209,8 @@ fs_download_started (struct FSUI_DownloadList *fsui_dl,
     {
       gtk_tree_store_append (download_summary, &iter, NULL);
     }
-  size = ECRS_fileSize (fi->uri);
-  size_h = string_get_fancy_byte_size (size);
+  size = GNUNET_ECRS_uri_get_file_size (fi->uri);
+  size_h = GNUNET_get_byte_size_as_fancy_string (size);
   sname = &filename[strlen (filename) - 1];
   while ((sname > filename) && (sname[-1] != '/') && (sname[-1] != '\\'))
     sname--;
@@ -213,7 +218,7 @@ fs_download_started (struct FSUI_DownloadList *fsui_dl,
     progress = completed * 100 / size;
   else
     progress = 100;
-  uri_name = ECRS_uriToString (fi->uri);
+  uri_name = GNUNET_ECRS_uri_to_string (fi->uri);
   gtk_tree_store_set (download_summary,
                       &iter,
                       DOWNLOAD_FILENAME, filename,
@@ -222,9 +227,11 @@ fs_download_started (struct FSUI_DownloadList *fsui_dl,
                       DOWNLOAD_HSIZE, size_h,
                       DOWNLOAD_PROGRESS, progress,
                       DOWNLOAD_URISTRING, uri_name,
-                      DOWNLOAD_INTERNAL, list, -1);
-  FREE (uri_name);
-  FREE (size_h);
+                      DOWNLOAD_INTERNAL, list,
+		      DOWNLOAD_META_DATA, GNUNET_meta_data_duplicate(fi->meta),
+		      -1);
+  GNUNET_free (uri_name);
+  GNUNET_free (size_h);
   path = gtk_tree_model_get_path (GTK_TREE_MODEL (download_summary), &iter);
   list->summaryViewRowReference
     = gtk_tree_row_reference_new (GTK_TREE_MODEL (download_summary), path);
@@ -241,27 +248,26 @@ fs_download_started (struct FSUI_DownloadList *fsui_dl,
           /* FIXME: this requires GTK 2.8. Since it doesn't support Win9x, the quick
              solution is to #ifndef it */
 #ifndef MINGW
-          GE_BREAK (ectx,
-                    gtk_tree_row_reference_get_model (dl_parent->
-                                                      searchViewRowReference)
-                    == model);
+          GNUNET_GE_BREAK (ectx,
+                           gtk_tree_row_reference_get_model
+                           (dl_parent->searchViewRowReference) == model);
 #endif
           path =
-            gtk_tree_row_reference_get_path (dl_parent->
-                                             searchViewRowReference);
+            gtk_tree_row_reference_get_path
+            (dl_parent->searchViewRowReference);
           if (path != NULL)
             {
               valid = gtk_tree_model_get_iter (model, &piter, path);
-              GE_BREAK (ectx, valid == TRUE);
+              GNUNET_GE_BREAK (ectx, valid == TRUE);
               if (valid == TRUE)
                 {
                   valid = gtk_tree_model_iter_children (model, &iter, &piter);
-                  GE_BREAK (ectx, valid == TRUE);
+                  GNUNET_GE_BREAK (ectx, valid == TRUE);
                 }
             }
           else
             {
-              GE_BREAK (ectx, 0);
+              GNUNET_GE_BREAK (ectx, 0);
               valid = FALSE;
             }
         }
@@ -269,23 +275,25 @@ fs_download_started (struct FSUI_DownloadList *fsui_dl,
         {
           /* no download-parent, must be top-level entry in search */
           valid = gtk_tree_model_get_iter_first (model, &iter);
-          GE_BREAK (ectx, valid == TRUE);
+          GNUNET_GE_BREAK (ectx, valid == TRUE);
         }
       while (valid == TRUE)
         {
           /* find matching entry */
           gtk_tree_model_get (model, &iter, SEARCH_URI, &u, -1);
-          if (ECRS_equalsUri (u, fi->uri))
+          if (GNUNET_ECRS_uri_test_equal (u, fi->uri))
             {
               path = gtk_tree_model_get_path (model, &iter);
               list->searchViewRowReference
                 = gtk_tree_row_reference_new (model, path);
               gtk_tree_path_free (path);
               gtk_tree_store_set (sl_parent->tree,
-                                  &iter,
-                                  SEARCH_CELL_BG_COLOR,
-                                  getColorCode (URITRACK_DOWNLOAD_STARTED),
-                                  -1);
+                                  &iter, SEARCH_STATUS,
+                                  getStatusName
+                                  (GNUNET_URITRACK_DOWNLOAD_STARTED),
+                                  SEARCH_STATUS_LOGO,
+                                  getStatusLogo
+                                  (GNUNET_URITRACK_DOWNLOAD_STARTED), -1);
               break;
             }
           valid = gtk_tree_model_iter_next (model, &iter);
@@ -294,17 +302,18 @@ fs_download_started (struct FSUI_DownloadList *fsui_dl,
         {
           /* did not find matching entry in search list -- bug!  Continue
              without adding to to search list! */
-          GE_BREAK (ectx, 0);
+          GNUNET_GE_BREAK (ectx, 0);
           list->searchList = NULL;
         }
     }
   list->fsui_list = fsui_dl;
   list->total = total;
-  list->is_directory = ECRS_isDirectory (fi->meta);
-  list->has_terminated = ((state != FSUI_ACTIVE) && (state != FSUI_PENDING));
+  list->is_directory = GNUNET_meta_data_test_for_directory (fi->meta);
+  list->has_terminated = ((state != GNUNET_FSUI_ACTIVE)
+                          && (state != GNUNET_FSUI_PENDING));
   list->next = download_head;
   download_head = list;
-  if ((list->is_directory == YES) && (completed != 0))
+  if ((list->is_directory == GNUNET_YES) && (completed != 0))
     refreshDirectoryViewFromDisk (list);
   return list;
 }
@@ -322,12 +331,12 @@ fs_download_update (DownloadList * list,
   GtkTreeIter iter;
   GtkTreePath *path;
   unsigned int val;
-  struct ECRS_MetaData *meta;
+  struct GNUNET_MetaData *meta;
 
   path = gtk_tree_row_reference_get_path (list->summaryViewRowReference);
   if (path == NULL)
     {
-      GE_BREAK (ectx, 0);
+      GNUNET_GE_BREAK (ectx, 0);
       return;
     }
   gtk_tree_model_get_iter (GTK_TREE_MODEL (download_summary), &iter, path);
@@ -337,14 +346,15 @@ fs_download_update (DownloadList * list,
   else
     val = 100;
   gtk_tree_store_set (download_summary, &iter, DOWNLOAD_PROGRESS, val, -1);
-  if ((list->is_directory == YES) &&
+  if ((list->is_directory == GNUNET_YES) &&
       (list->searchList != NULL) && (list->searchViewRowReference != NULL))
     {
       meta = NULL;
-      ECRS_listDirectory (ectx,
-                          data, size, &meta, &addFilesToDirectory, list);
+      GNUNET_ECRS_directory_list_contents (ectx,
+                                           data, size, NULL, &meta,
+                                           &addFilesToDirectory, list);
       if (meta != NULL)
-        ECRS_freeMetaData (meta);
+        GNUNET_meta_data_destroy (meta);
     }
 }
 
@@ -361,18 +371,21 @@ fs_download_completed (DownloadList * downloadContext)
   if (downloadContext->searchViewRowReference != NULL)
     {
       path =
-        gtk_tree_row_reference_get_path (downloadContext->
-                                         searchViewRowReference);
+        gtk_tree_row_reference_get_path
+        (downloadContext->searchViewRowReference);
       gtk_tree_model_get_iter (GTK_TREE_MODEL
                                (downloadContext->searchList->tree), &iter,
                                path);
       gtk_tree_path_free (path);
       gtk_tree_store_set (downloadContext->searchList->tree,
                           &iter,
-                          SEARCH_CELL_BG_COLOR,
-                          getColorCode (URITRACK_DOWNLOAD_COMPLETED), -1);
+                          SEARCH_STATUS,
+                          getStatusName (GNUNET_URITRACK_DOWNLOAD_COMPLETED),
+                          SEARCH_STATUS_LOGO,
+                          getStatusLogo (GNUNET_URITRACK_DOWNLOAD_COMPLETED),
+                          -1);
     }
-  downloadContext->has_terminated = YES;
+  downloadContext->has_terminated = GNUNET_YES;
   refreshDirectoryViewFromDisk (downloadContext);
 }
 
@@ -389,21 +402,26 @@ fs_download_aborted (DownloadList * downloadContext)
   if (downloadContext->searchViewRowReference != NULL)
     {
       path =
-        gtk_tree_row_reference_get_path (downloadContext->
-                                         searchViewRowReference);
+        gtk_tree_row_reference_get_path
+        (downloadContext->searchViewRowReference);
       gtk_tree_model_get_iter (GTK_TREE_MODEL
                                (downloadContext->searchList->tree), &iter,
                                path);
       gtk_tree_path_free (path);
       gtk_tree_store_set (downloadContext->searchList->tree,
                           &iter,
-                          SEARCH_CELL_BG_COLOR,
-                          getColorCode (URITRACK_DOWNLOAD_ABORTED), -1);
+                          SEARCH_STATUS,
+                          getStatusName (GNUNET_URITRACK_DOWNLOAD_ABORTED),
+                          SEARCH_STATUS_LOGO,
+                          getStatusLogo (GNUNET_URITRACK_DOWNLOAD_ABORTED),
+                          -1);
     }
-  downloadContext->has_terminated = YES;
+  downloadContext->has_terminated = GNUNET_YES;
   refreshDirectoryViewFromDisk (downloadContext);
 }
 
+
+
 /**
  * A download has been stopped.  Remove from summary
  * and free associated resources.
@@ -412,16 +430,16 @@ void
 fs_download_stopped (DownloadList * list)
 {
   GtkTreeIter iter;
-  GtkTreeIter piter;
+  GtkTreeIter citer;
   GtkTreePath *path;
   DownloadList *prev;
-  int valid;
   GtkTreeModel *model;
+  struct GNUNET_MetaData * meta;
 
   path = gtk_tree_row_reference_get_path (list->summaryViewRowReference);
   if (path == NULL)
     {
-      GE_BREAK (ectx, 0);
+      GNUNET_GE_BREAK (ectx, 0);
     }
   else
     {
@@ -430,27 +448,46 @@ fs_download_stopped (DownloadList * list)
       gtk_tree_path_free (path);
       gtk_tree_row_reference_free (list->summaryViewRowReference);
       list->summaryViewRowReference = NULL;
+      gtk_tree_model_get(GTK_TREE_MODEL(download_summary),
+			 &iter,
+			 DOWNLOAD_META_DATA, &meta,
+			 -1);
+      if (meta != NULL)
+	GNUNET_meta_data_destroy(meta);		 
       gtk_tree_store_remove (download_summary, &iter);
     }
-  FREE (list->filename);
-  ECRS_freeUri (list->uri);
+  GNUNET_free (list->filename);
+  GNUNET_ECRS_uri_destroy (list->uri);
 
-  /* if we have child-results in view, remove them! */
-  if (list->searchList != NULL)
+  if ((list->searchList != NULL) && (list->searchViewRowReference != NULL))
     {
       path = gtk_tree_row_reference_get_path (list->searchViewRowReference);
       if (path == NULL)
         {
-          GE_BREAK (ectx, 0);
+          GNUNET_GE_BREAK (ectx, 0);
         }
       else
         {
           model = GTK_TREE_MODEL (list->searchList->tree);
-          gtk_tree_model_get_iter (model, &piter, path);
+          gtk_tree_model_get_iter (model, &iter, path);
           gtk_tree_path_free (path);
-          valid = gtk_tree_model_iter_children (model, &iter, &piter);
-          while (TRUE == valid)
-            valid = gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
+          gtk_tree_store_set (list->searchList->tree,
+                              &iter,
+                              SEARCH_STATUS,
+                              getStatusName
+                              (GNUNET_URITRACK_DOWNLOAD_ABORTED),
+                              SEARCH_STATUS_LOGO,
+                              getStatusLogo
+                              (GNUNET_URITRACK_DOWNLOAD_ABORTED), -1);
+	  if (gtk_tree_model_iter_children (model, &citer, &iter))
+	    {
+	      do
+		{
+		  gtk_tree_store_set (list->searchList->tree, 
+				      &citer, SEARCH_INTERNAL_PARENT, NULL, -1);
+		}
+	      while (gtk_tree_model_iter_next (model, &citer));
+	    }
         }
     }
   if (list->searchViewRowReference != NULL)
@@ -471,9 +508,9 @@ fs_download_stopped (DownloadList * list)
       if (prev != NULL)
         prev->next = list->next;
       else
-        GE_BREAK (ectx, 0);
+        GNUNET_GE_BREAK (ectx, 0);
     }
-  FREE (list);
+  GNUNET_free (list);
 }
 
 
@@ -483,7 +520,7 @@ fs_download_stopped (DownloadList * list)
  * Check if a download for the given filename is
  * already running.
  *
- * @return OK if no download is pending, SYSERR if
+ * @return GNUNET_OK if no download is pending, GNUNET_SYSERR if
  *  such a download is already active.
  */
 static int
@@ -502,23 +539,23 @@ check_pending (const char *filename, GtkTreeIter * parent)
           if ((name != NULL) && (0 == strcmp (name, filename)))
             {
               free (name);
-              return SYSERR;
+              return GNUNET_SYSERR;
             }
           if (name != NULL)
             free (name);
-          if (SYSERR == check_pending (filename, &iter))
-            return SYSERR;
+          if (GNUNET_SYSERR == check_pending (filename, &iter))
+            return GNUNET_SYSERR;
         }
       while (gtk_tree_model_iter_next (model, &iter));
     }
-  return OK;
+  return GNUNET_OK;
 }
 
 typedef struct
 {
   char *uri_name;
-  struct ECRS_URI *idc_uri;
-  struct ECRS_MetaData *idc_meta;
+  struct GNUNET_ECRS_URI *idc_uri;
+  struct GNUNET_MetaData *idc_meta;
   char *idc_final_download_destination;
   SearchList *searchContext;
   DownloadList *parentContext;
@@ -531,16 +568,16 @@ init_download_helper (void *cls)
 {
   SDC *sdc = cls;
 
-  FSUI_startDownload (ctx,
-                      sdc->anonymity,
-                      sdc->recursive,
-                      sdc->idc_uri,
-                      sdc->idc_meta,
-                      sdc->idc_final_download_destination,
-                      (sdc->searchContext !=
-                       NULL) ? sdc->searchContext->fsui_list : NULL,
-                      (sdc->parentContext !=
-                       NULL) ? sdc->parentContext->fsui_list : NULL);
+  GNUNET_FSUI_download_start (ctx,
+                              sdc->anonymity,
+                              sdc->recursive,
+                              sdc->idc_uri,
+                              sdc->idc_meta,
+                              sdc->idc_final_download_destination,
+                              (sdc->searchContext !=
+                               NULL) ? sdc->searchContext->fsui_list : NULL,
+                              (sdc->parentContext !=
+                               NULL) ? sdc->parentContext->fsui_list : NULL);
   return NULL;
 }
 
@@ -562,12 +599,10 @@ initiateDownload (GtkTreeModel * model,
   char *dirPath;
   unsigned int dirPathLen;
   char *idc_name;
-  char *idc_mime;
 
   sdc.idc_uri = NULL;
   sdc.idc_meta = NULL;
   idc_name = NULL;
-  idc_mime = NULL;
   sdc.searchContext = NULL;
   sdc.parentContext = NULL;
   gtk_tree_model_get (model,
@@ -575,26 +610,24 @@ initiateDownload (GtkTreeModel * model,
                       SEARCH_NAME, &idc_name,
                       SEARCH_URI, &sdc.idc_uri,
                       SEARCH_META, &sdc.idc_meta,
-                      SEARCH_MIME, &idc_mime,
                       SEARCH_INTERNAL, &sdc.searchContext,
                       SEARCH_INTERNAL_PARENT, &sdc.parentContext, -1);
   if ((sdc.idc_uri == NULL) ||
-      (!(ECRS_isFileUri (sdc.idc_uri) || ECRS_isLocationUri (sdc.idc_uri))))
+      (!(GNUNET_ECRS_uri_test_chk (sdc.idc_uri)
+         || GNUNET_ECRS_uri_test_loc (sdc.idc_uri))))
     {
-      GE_BREAK (ectx, 0);
-      FREENONNULL (idc_name);
-      FREENONNULL (idc_mime);
+      GNUNET_GE_BREAK (ectx, 0);
+      GNUNET_free_non_null (idc_name);
       return;
     }
-  sdc.uri_name = ECRS_uriToString (sdc.idc_uri);
+  sdc.uri_name = GNUNET_ECRS_uri_to_string (sdc.idc_uri);
   if ((sdc.uri_name == NULL) ||
       (strlen (sdc.uri_name) <
-       strlen (ECRS_URI_PREFIX) + strlen (ECRS_FILE_INFIX)))
+       strlen (GNUNET_ECRS_URI_PREFIX) + strlen (GNUNET_ECRS_FILE_INFIX)))
     {
-      GE_BREAK (ectx, 0);
-      FREENONNULL (sdc.uri_name);
-      FREENONNULL (idc_name);
-      FREENONNULL (idc_mime);
+      GNUNET_GE_BREAK (ectx, 0);
+      GNUNET_free_non_null (sdc.uri_name);
+      GNUNET_free_non_null (idc_name);
       return;
     }
   /* reduce "//" to "/" */
@@ -611,32 +644,32 @@ initiateDownload (GtkTreeModel * model,
 #ifdef WINDOWS
       char *filehash;
 
-      GE_ASSERT (NULL,
-                 strlen (sdc.uri_name) >
-                 strlen (ECRS_URI_PREFIX) + strlen (ECRS_FILE_INFIX));
-      FREENONNULL (idc_name);
+      GNUNET_GE_ASSERT (NULL,
+                        strlen (sdc.uri_name) >
+                        strlen (GNUNET_ECRS_URI_PREFIX) +
+                        strlen (GNUNET_ECRS_FILE_INFIX));
+      GNUNET_free_non_null (idc_name);
       filehash =
-        STRDUP (&sdc.
-                uri_name[strlen (ECRS_URI_PREFIX) +
-                         strlen (ECRS_FILE_INFIX)]);
+        GNUNET_strdup (&sdc.uri_name[strlen (GNUNET_ECRS_URI_PREFIX) +
+                                     strlen (GNUNET_ECRS_FILE_INFIX)]);
       filehash[16] = 0;
-      idc_name = STRDUP (filehash);
-      FREENONNULL (filehash);
+      idc_name = GNUNET_strdup (filehash);
+      GNUNET_free_non_null (filehash);
 #else
-      GE_ASSERT (NULL,
-                 strlen (sdc.uri_name) >
-                 strlen (ECRS_URI_PREFIX) + strlen (ECRS_FILE_INFIX));
-      FREENONNULL (idc_name);
+      GNUNET_GE_ASSERT (NULL,
+                        strlen (sdc.uri_name) >
+                        strlen (GNUNET_ECRS_URI_PREFIX) +
+                        strlen (GNUNET_ECRS_FILE_INFIX));
+      GNUNET_free_non_null (idc_name);
       idc_name =
-        STRDUP (&sdc.
-                uri_name[strlen (ECRS_URI_PREFIX) +
-                         strlen (ECRS_FILE_INFIX)]);
+        GNUNET_strdup (&sdc.uri_name[strlen (GNUNET_ECRS_URI_PREFIX) +
+                                     strlen (GNUNET_ECRS_FILE_INFIX)]);
 #endif
     }
 
   /* dname = directory portion of idc_name */
   cname = idc_name;
-  dname = STRDUP (idc_name);
+  dname = GNUNET_strdup (idc_name);
   cname = &dname[strlen (dname) - 1];
   if (cname != dname)
     cname--;                    /* ignore tailing '/' */
@@ -645,8 +678,8 @@ initiateDownload (GtkTreeModel * model,
   if (*cname == DIR_SEPARATOR)
     {
       *cname = '\0';
-      FREE (idc_name);
-      idc_name = STRDUP (cname + 1);
+      GNUNET_free (idc_name);
+      idc_name = GNUNET_strdup (cname + 1);
     }
   else
     {
@@ -654,14 +687,14 @@ initiateDownload (GtkTreeModel * model,
     }
   cname = NULL;
 
-  GC_get_configuration_value_filename (cfg,
-                                       "FS",
-                                       "INCOMINGDIR",
-                                       "$HOME/gnunet-downloads/",
-                                       &final_download_dir);
+  GNUNET_GC_get_configuration_value_filename (cfg,
+                                              "FS",
+                                              "INCOMINGDIR",
+                                              "$HOME/gnunet-downloads/",
+                                              &final_download_dir);
   if (strlen (dname) > 0)
     {
-      tmp = MALLOC (strlen (final_download_dir) + strlen (dname) + 2);
+      tmp = GNUNET_malloc (strlen (final_download_dir) + strlen (dname) + 2);
       strcpy (tmp, final_download_dir);
       if (tmp[strlen (tmp)] != DIR_SEPARATOR)
         strcat (tmp, DIR_SEPARATOR_STR);
@@ -669,14 +702,14 @@ initiateDownload (GtkTreeModel * model,
         strcat (tmp, &dname[1]);
       else
         strcat (tmp, dname);
-      FREE (final_download_dir);
+      GNUNET_free (final_download_dir);
       final_download_dir = tmp;
     }
-  FREE (dname);
+  GNUNET_free (dname);
   dname = NULL;
   /* If file is inside a directory, get the full path */
   dirTreePath = gtk_tree_path_copy (path);
-  dirPath = MALLOC (1);
+  dirPath = GNUNET_malloc (1);
   dirPath[0] = '\0';
   dirPathLen = 0;
   while (gtk_tree_path_get_depth (dirTreePath) > 1)
@@ -691,12 +724,12 @@ initiateDownload (GtkTreeModel * model,
       gtk_tree_model_get (model, &iiter, SEARCH_NAME, &dirname, -1);
       dirPathLen =
         strlen (dirPath) + strlen (dirname) + strlen (DIR_SEPARATOR_STR) + 1;
-      newPath = MALLOC (dirPathLen + 1);
+      newPath = GNUNET_malloc (dirPathLen + 1);
       strcpy (newPath, dirname);
       if (newPath[strlen (newPath) - 1] != DIR_SEPARATOR)
         strcat (newPath, DIR_SEPARATOR_STR);
       strcat (newPath, dirPath);
-      FREE (dirPath);
+      GNUNET_free (dirPath);
       dirPath = newPath;
       free (dirname);
     }
@@ -704,13 +737,12 @@ initiateDownload (GtkTreeModel * model,
 
   /* construct completed/directory/real-filename */
   sdc.idc_final_download_destination =
-    MALLOC (strlen (final_download_dir) + 2 + strlen (idc_name) +
-            strlen (GNUNET_DIRECTORY_EXT) + strlen (dirPath));
+    GNUNET_malloc (strlen (final_download_dir) + 2 + strlen (idc_name) +
+                   strlen (GNUNET_DIRECTORY_EXT) + strlen (dirPath));
   strcpy (sdc.idc_final_download_destination, final_download_dir);
-  if (sdc.
-      idc_final_download_destination[strlen
-                                     (sdc.idc_final_download_destination) -
-                                     1] != DIR_SEPARATOR)
+  if (sdc.idc_final_download_destination[strlen
+                                         (sdc.idc_final_download_destination)
+                                         - 1] != DIR_SEPARATOR)
     strcat (sdc.idc_final_download_destination, DIR_SEPARATOR_STR);
   strcat (sdc.idc_final_download_destination, dirPath);
   strcat (sdc.idc_final_download_destination, idc_name);
@@ -718,26 +750,26 @@ initiateDownload (GtkTreeModel * model,
                                       "downloadAnonymitySpinButton");
   sdc.recursive = getToggleButtonValue (sdc.searchContext->searchXML,
                                         "downloadRecursiveCheckButton");
-  if (OK == check_pending (idc_name, NULL))
+  if (GNUNET_OK == check_pending (idc_name, NULL))
     {
-      addLogEntry (_("Downloading `%s'\n"), idc_name);
-      run_with_save_calls (&init_download_helper, &sdc);
+      GNUNET_GTK_add_log_entry (_("Downloading `%s'\n"), idc_name);
+      GNUNET_GTK_run_with_save_calls (&init_download_helper, &sdc);
     }
   else
     {
-      addLogEntry (_("ERROR: already downloading `%s'"), idc_name);
+      GNUNET_GTK_add_log_entry (_("ERROR: already downloading `%s'"),
+                                idc_name);
     }
-  FREE (sdc.uri_name);
-  FREE (dirPath);
-  FREE (sdc.idc_final_download_destination);
-  FREENONNULL (final_download_dir);
-  FREENONNULL (idc_name);
-  FREENONNULL (idc_mime);
+  GNUNET_free (sdc.uri_name);
+  GNUNET_free (dirPath);
+  GNUNET_free (sdc.idc_final_download_destination);
+  GNUNET_free_non_null (final_download_dir);
+  GNUNET_free_non_null (idc_name);
 }
 
 /**
  * The download button in the search dialog was
- * clicked.  Download all selected entries.
+ * clicked.  GNUNET_ND_DOWNLOAD all selected entries.
  */
 void
 on_downloadButton_clicked_fs (GtkWidget * treeview,
@@ -746,7 +778,20 @@ on_downloadButton_clicked_fs (GtkWidget * treeview,
   GtkTreeSelection *selection;
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
-  ggc_tree_selection_selected_foreach (selection, &initiateDownload, NULL);
+  GNUNET_GTK_tree_selection_selected_foreach (selection, &initiateDownload,
+                                              NULL);
+}
+
+/**
+ * Double-click on a search result
+ */
+gboolean
+on_searchResults_button_press_fs (GtkWidget * treeview,
+                                  GdkEventButton * event, gpointer data)
+{
+  if (event->type == GDK_2BUTTON_PRESS)
+    on_downloadButton_clicked_fs (treeview, NULL);
+  return FALSE;
 }
 
 
@@ -770,71 +815,73 @@ on_statusDownloadURIEntry_editing_done_fs (GtkWidget * entry,
   SDC sdc;
 
   uris = gtk_entry_get_text (GTK_ENTRY (entry));
-  urid = STRDUP (uris);
-  gtk_entry_set_text (GTK_ENTRY (entry), ECRS_URI_PREFIX);
-  sdc.idc_uri = ECRS_stringToUri (ectx, urid);
+  urid = GNUNET_strdup (uris);
+  gtk_entry_set_text (GTK_ENTRY (entry), GNUNET_ECRS_URI_PREFIX);
+  sdc.idc_uri = GNUNET_ECRS_string_to_uri (ectx, urid);
   if (sdc.idc_uri == NULL)
     {
-      addLogEntry (_("Invalid URI `%s'"), urid);
-      FREE (urid);
+      GNUNET_GTK_add_log_entry (_("Invalid URI `%s'"), urid);
+      GNUNET_free (urid);
       return;
     }
-  if (ECRS_isKeywordUri (sdc.idc_uri))
+  if (GNUNET_ECRS_uri_test_ksk (sdc.idc_uri))
     {
-      addLogEntry (_
-                   ("Please use the search function for keyword (KSK) URIs!"));
-      FREE (urid);
-      ECRS_freeUri (sdc.idc_uri);
+      GNUNET_GTK_add_log_entry (_
+                                ("Please use the search function for keyword (KSK) URIs!"));
+      GNUNET_free (urid);
+      GNUNET_ECRS_uri_destroy (sdc.idc_uri);
       return;
     }
-  else if (ECRS_isLocationUri (sdc.idc_uri))
+  else if (GNUNET_ECRS_uri_test_loc (sdc.idc_uri))
     {
-      addLogEntry (_("Location URIs are not yet supported"));
-      FREE (urid);
-      ECRS_freeUri (sdc.idc_uri);
+      GNUNET_GTK_add_log_entry (_("Location URIs are not yet supported"));
+      GNUNET_free (urid);
+      GNUNET_ECRS_uri_destroy (sdc.idc_uri);
       return;
     }
-  GC_get_configuration_value_filename (cfg,
-                                       "FS",
-                                       "INCOMINGDIR",
-                                       "$HOME/gnunet-downloads/",
-                                       &final_download_dir);
-  disk_directory_create (ectx, final_download_dir);
-  dname = &uris[strlen (ECRS_URI_PREFIX) + strlen (ECRS_FILE_INFIX)];
+  GNUNET_GC_get_configuration_value_filename (cfg,
+                                              "FS",
+                                              "INCOMINGDIR",
+                                              "$HOME/gnunet-downloads/",
+                                              &final_download_dir);
+  GNUNET_disk_directory_create (ectx, final_download_dir);
+  dname =
+    &uris[strlen (GNUNET_ECRS_URI_PREFIX) + strlen (GNUNET_ECRS_FILE_INFIX)];
   sdc.idc_final_download_destination =
-    MALLOC (strlen (final_download_dir) + strlen (dname) + 2);
+    GNUNET_malloc (strlen (final_download_dir) + strlen (dname) + 2);
   strcpy (sdc.idc_final_download_destination, final_download_dir);
-  FREE (final_download_dir);
-  if (sdc.
-      idc_final_download_destination[strlen
-                                     (sdc.idc_final_download_destination)] !=
-      DIR_SEPARATOR)
+  GNUNET_free (final_download_dir);
+  if (sdc.idc_final_download_destination[strlen
+                                         (sdc.idc_final_download_destination)]
+      != DIR_SEPARATOR)
     strcat (sdc.idc_final_download_destination, DIR_SEPARATOR_STR);
   strcat (sdc.idc_final_download_destination, dname);
 
-  addLogEntry (_("Downloading `%s'\n"), uris);
-  sdc.idc_meta = ECRS_createMetaData ();
-  sdc.anonymity = getSpinButtonValue (getMainXML (), "fsstatusAnonymitySpin");
-  sdc.recursive = NO;
+  GNUNET_GTK_add_log_entry (_("Downloading `%s'\n"), uris);
+  sdc.idc_meta = GNUNET_meta_data_create ();
+  sdc.anonymity =
+    getSpinButtonValue (GNUNET_GTK_get_main_glade_XML (),
+                        "fsstatusAnonymitySpin");
+  sdc.recursive = GNUNET_NO;
   sdc.searchContext = NULL;
   sdc.parentContext = NULL;
-  run_with_save_calls (&init_download_helper, &sdc);
-  ECRS_freeMetaData (sdc.idc_meta);
-  FREE (sdc.idc_final_download_destination);
-  FREE (urid);
+  GNUNET_GTK_run_with_save_calls (&init_download_helper, &sdc);
+  GNUNET_meta_data_destroy (sdc.idc_meta);
+  GNUNET_free (sdc.idc_final_download_destination);
+  GNUNET_free (urid);
 }
 
 struct FCBC
 {
-  int (*method) (struct FSUI_Context * ctx, struct FSUI_DownloadList * list);
-  struct FSUI_DownloadList *argument;
+  int (*method) (struct GNUNET_FSUI_DownloadList * list);
+  struct GNUNET_FSUI_DownloadList *argument;
 };
 
 static void *
 fsui_callback (void *cls)
 {
   struct FCBC *fcbc = cls;
-  fcbc->method (ctx, fcbc->argument);
+  fcbc->method (fcbc->argument);
   return NULL;
 }
 
@@ -846,23 +893,22 @@ clearCompletedDownloadCallback (GtkTreeModel * model,
   DownloadList *dl;
   struct FCBC fcbc;
 
-  GE_ASSERT (ectx, model == GTK_TREE_MODEL (download_summary));
+  GNUNET_GE_ASSERT (ectx, model == GTK_TREE_MODEL (download_summary));
   gtk_tree_model_get (model, iter, DOWNLOAD_INTERNAL, &dl, -1);
   if ((FALSE == gtk_tree_model_iter_has_child (model,
                                                iter)) && (dl->has_terminated))
     {
-      fcbc.method = &FSUI_stopDownload;
+      fcbc.method = &GNUNET_FSUI_download_stop;
       fcbc.argument = dl->fsui_list;
-      run_with_save_calls (&fsui_callback, &fcbc);
+      GNUNET_GTK_run_with_save_calls (&fsui_callback, &fcbc);
     }
 }
 
 void
-on_clearCompletedDownloadsButton_clicked_fs (void *unused,
-                                             GtkWidget * clearButton)
+on_clearCompletedDownloads_clicked_fs (void *unused, GtkWidget * clearButton)
 {
-  ggc_tree_model_foreach (GTK_TREE_MODEL (download_summary),
-                          &clearCompletedDownloadCallback, NULL);
+  GNUNET_GTK_tree_model_foreach (GTK_TREE_MODEL (download_summary),
+                                 &clearCompletedDownloadCallback, NULL);
 }
 
 static void
@@ -873,37 +919,99 @@ fsuiCallDownloadCallback (GtkTreeModel * model,
   DownloadList *dl;
   struct FCBC fcbc;
 
-  GE_ASSERT (ectx, model == GTK_TREE_MODEL (download_summary));
+  GNUNET_GE_ASSERT (ectx, model == GTK_TREE_MODEL (download_summary));
   gtk_tree_model_get (model, iter, DOWNLOAD_INTERNAL, &dl, -1);
   fcbc.method = fsui_call;
   fcbc.argument = dl->fsui_list;
-  run_with_save_calls (&fsui_callback, &fcbc);
+  GNUNET_GTK_run_with_save_calls (&fsui_callback, &fcbc);
 }
 
 void
-on_abortDownloadButton_clicked_fs (void *unused, GtkWidget * abortButton)
+on_abortDownload_clicked_fs (void *unused, GtkWidget * dummy)
 {
   GtkTreeSelection *selection;
   GtkWidget *downloadList;
 
-  downloadList = glade_xml_get_widget (getMainXML (), "activeDownloadsList");
+  downloadList =
+    glade_xml_get_widget (GNUNET_GTK_get_main_glade_XML (),
+                          "activeDownloadsList");
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (downloadList));
-  ggc_tree_selection_selected_foreach
-    (selection, &fsuiCallDownloadCallback, &FSUI_abortDownload);
+  GNUNET_GTK_tree_selection_selected_foreach
+    (selection, &fsuiCallDownloadCallback, &GNUNET_FSUI_download_abort);
+  GNUNET_GTK_tree_selection_selected_foreach
+    (selection, &fsuiCallDownloadCallback, &GNUNET_FSUI_download_stop);
 }
 
 void
-on_stopDownloadButton_clicked_fs (void *unused, GtkWidget * stopButton)
+on_stopDownload_clicked_fs (void *unused, GtkWidget * dummy)
 {
   GtkTreeSelection *selection;
   GtkWidget *downloadList;
 
-  downloadList = glade_xml_get_widget (getMainXML (), "activeDownloadsList");
+  downloadList =
+    glade_xml_get_widget (GNUNET_GTK_get_main_glade_XML (),
+                          "activeDownloadsList");
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (downloadList));
-  ggc_tree_selection_selected_foreach
-    (selection, &fsuiCallDownloadCallback, &FSUI_abortDownload);
-  ggc_tree_selection_selected_foreach
-    (selection, &fsuiCallDownloadCallback, &FSUI_stopDownload);
+  GNUNET_GTK_tree_selection_selected_foreach
+    (selection, &fsuiCallDownloadCallback, &GNUNET_FSUI_download_stop);
+}
+
+
+static void
+fsuiShowMetaDataCallback (GtkTreeModel * model,
+                          GtkTreePath * path,
+                          GtkTreeIter * iter, gpointer fsui_call)
+{
+  struct GNUNET_MetaData * meta;
+  char * name;
+
+  GNUNET_GE_ASSERT (ectx,
+		    model == GTK_TREE_MODEL (download_summary));
+  gtk_tree_model_get (model,
+		      iter, 
+		      DOWNLOAD_META_DATA, &meta,
+		      DOWNLOAD_SHORTNAME, &name,
+		      -1);
+  if (meta != NULL)
+    open_meta_data_display_dialog(meta, name);
+  GNUNET_free_non_null(name);
+}
+
+void
+on_showDownloadMetaData_clicked_fs (void *unused, GtkWidget * dummy)
+{
+  GtkTreeSelection *selection;
+  GtkWidget *downloadList;
+
+  downloadList =
+    glade_xml_get_widget (GNUNET_GTK_get_main_glade_XML (),
+                          "activeDownloadsList");
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (downloadList));
+  GNUNET_GTK_tree_selection_selected_foreach
+    (selection, &fsuiShowMetaDataCallback, NULL);
+}
+
+/**
+ * Right-click on an active download
+ */
+gboolean
+on_activeDownloadsList_button_press_fs (GtkWidget * treeview,
+                                        GdkEventButton * event_button,
+                                        gpointer dummy)
+{
+  GtkWidget *menu;
+  GladeXML *contextMenuXML;
+
+  contextMenuXML =
+    glade_xml_new (GNUNET_GTK_get_glade_filename (),
+                   "downloadsContextMenu", PACKAGE_NAME);
+  GNUNET_GTK_connect_glade_with_plugins (contextMenuXML);
+  menu = glade_xml_get_widget (contextMenuXML, "downloadsContextMenu");
+  if (event_button->button == 3)
+    gtk_menu_popup (GTK_MENU (menu),
+                    NULL, NULL, NULL, NULL,
+                    event_button->button, event_button->time);
+  return FALSE;
 }
 
 /* end of download.c */
