@@ -38,6 +38,8 @@ publishMetaData (EXTRACTOR_KeywordType type, const char *data, void *cls)
   const char *stype;
   GtkTreeIter iter;
 
+  if (EXTRACTOR_isBinaryType (type))
+    return GNUNET_OK;
   stype = EXTRACTOR_getKeywordTypeAsString (type);
   if (stype == NULL)            /* wrong LE version!? */
     stype = _("unknown");
@@ -46,14 +48,14 @@ publishMetaData (EXTRACTOR_KeywordType type, const char *data, void *cls)
                       &iter,
                       META_TYPE, type,
                       META_STYPE, stype, META_VALUE, data, -1);
-  return OK;
+  return GNUNET_OK;
 }
 
 void
 createMetaDataListTreeView (GladeXML * xml,
                             const char *name,
                             const char *previewName,
-                            const struct ECRS_MetaData *init)
+                            const struct GNUNET_MetaData *init)
 {
   GtkTreeView *metaList;
   GtkListStore *metamodel;
@@ -66,7 +68,7 @@ createMetaDataListTreeView (GladeXML * xml,
   GtkTreeViewColumn *column;
   int col;
 
-  DEBUG_BEGIN ();
+  GNUNET_GTK_DEBUG_BEGIN ();
   metamodel
     = gtk_list_store_new (META_NUM, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING);
   metaList = GTK_TREE_VIEW (glade_xml_get_widget (xml, name));
@@ -99,13 +101,13 @@ createMetaDataListTreeView (GladeXML * xml,
   gtk_tree_view_set_model (metaList, GTK_TREE_MODEL (metamodel));
 
   if (init != NULL)
-    ECRS_getMetaData (init, &publishMetaData, metamodel);
+    GNUNET_meta_data_get_contents (init, &publishMetaData, metamodel);
   if ((previewName == NULL) || (init == NULL))
     return;
   preview = glade_xml_get_widget (xml, previewName);
 
   thumb = NULL;
-  ts = ECRS_getThumbnailFromMetaData (init, &thumb);
+  ts = GNUNET_meta_data_get_thumbnail (init, &thumb);
   preview = glade_xml_get_widget (xml, previewName);
   if (ts != 0)
     {
@@ -121,32 +123,33 @@ createMetaDataListTreeView (GladeXML * xml,
         }
       UNREF (loader);
     }
-  DEBUG_END ();
+  GNUNET_GTK_DEBUG_END ();
 }
 
 static int
-publishKeyword (const char *data, void *cls)
+publishKeyword (const char *data, int is_mandatory, void *cls)
 {
   GtkListStore *store = cls;
   GtkTreeIter iter;
 
-  DEBUG_BEGIN ();
+  GNUNET_GTK_DEBUG_BEGIN ();
   gtk_list_store_append (store, &iter);
   gtk_list_store_set (store, &iter, 0, data, -1);
-  DEBUG_END ();
-  return OK;
+  GNUNET_GTK_DEBUG_END ();
+  return GNUNET_OK;
 }
 
 void
 createKeywordListTreeView (GladeXML * xml,
-                           const char *name, const struct ECRS_URI *init)
+                           const char *name,
+                           const struct GNUNET_ECRS_URI *init)
 {
   GtkTreeView *keywordList;
   GtkListStore *keymodel;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
 
-  DEBUG_BEGIN ();
+  GNUNET_GTK_DEBUG_BEGIN ();
   keymodel = gtk_list_store_new (1, G_TYPE_STRING);
   keywordList = GTK_TREE_VIEW (glade_xml_get_widget (xml, name));
   gtk_tree_selection_set_mode (gtk_tree_view_get_selection (keywordList),
@@ -163,8 +166,8 @@ createKeywordListTreeView (GladeXML * xml,
   gtk_tree_view_column_set_sort_column_id (column, 0);
   gtk_tree_view_set_model (keywordList, GTK_TREE_MODEL (keymodel));
   if (init != NULL)
-    ECRS_getKeywordsFromUri (init, &publishKeyword, keymodel);
-  DEBUG_END ();
+    GNUNET_ECRS_uri_get_keywords_from_ksk (init, &publishKeyword, keymodel);
+  GNUNET_GTK_DEBUG_END ();
 }
 
 void
@@ -177,7 +180,7 @@ createMetaTypeComboBox (GladeXML * xml, const char *name)
   GtkTreeIter iter;
   EXTRACTOR_KeywordType type;
 
-  DEBUG_BEGIN ();
+  GNUNET_GTK_DEBUG_BEGIN ();
   keywordTypeModel
     = gtk_list_store_new (KTYPE_NUM, G_TYPE_STRING, G_TYPE_INT);
   metaType = glade_xml_get_widget (xml, name);
@@ -185,6 +188,9 @@ createMetaTypeComboBox (GladeXML * xml, const char *name)
                            GTK_TREE_MODEL (keywordTypeModel));
   for (type = 0; type < EXTRACTOR_getHighestKeywordTypeNumber (); type++)
     {
+      if ((type == EXTRACTOR_SPLIT) ||
+          (type == EXTRACTOR_LOWERCASE) || (EXTRACTOR_isBinaryType (type)))
+        continue;               /* these are not legal... */
       stype = EXTRACTOR_getKeywordTypeAsString (type);
       gtk_list_store_append (keywordTypeModel, &iter);
       gtk_list_store_set (keywordTypeModel,
@@ -195,7 +201,7 @@ createMetaTypeComboBox (GladeXML * xml, const char *name)
   gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (metaType),
                                  renderer, "text", KTYPE_STRING);
   gtk_combo_box_set_active (GTK_COMBO_BOX (metaType), 0);
-  DEBUG_END ();
+  GNUNET_GTK_DEBUG_END ();
 }
 
 void
@@ -209,7 +215,7 @@ handleKeywordListUpdate (GladeXML * xml,
   GtkListStore *keymodel;
   GtkTreeIter iter;
 
-  DEBUG_BEGIN ();
+  GNUNET_GTK_DEBUG_BEGIN ();
   keywordList = glade_xml_get_widget (xml, keywordListName);
   keymodel
     = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (keywordList)));
@@ -219,7 +225,7 @@ handleKeywordListUpdate (GladeXML * xml,
   gtk_list_store_append (keymodel, &iter);
   gtk_list_store_set (keymodel, &iter, 0, keyword, -1);
   gtk_entry_set_text (GTK_ENTRY (entryLine), "");
-  DEBUG_END ();
+  GNUNET_GTK_DEBUG_END ();
 }
 
 void
@@ -238,7 +244,7 @@ handleMetaDataListUpdate (GladeXML * xml,
   GtkTreeIter iter;
   char *stype;
 
-  DEBUG_BEGIN ();
+  GNUNET_GTK_DEBUG_BEGIN ();
   metaList = glade_xml_get_widget (xml, metaDataListName);
   metamodel
     = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (metaList)));
@@ -260,8 +266,8 @@ handleMetaDataListUpdate (GladeXML * xml,
                       META_TYPE, type,
                       META_STYPE, stype, META_VALUE, value, -1);
   gtk_entry_set_text (GTK_ENTRY (entryLine), "");
-  FREENONNULL (stype);
-  DEBUG_END ();
+  GNUNET_free_non_null (stype);
+  GNUNET_GTK_DEBUG_END ();
 }
 
 static void
@@ -277,14 +283,14 @@ handleListRemove (GladeXML * xml, const char *listName)
   GtkWidget *list;
 
   list = glade_xml_get_widget (xml, listName);
-  ggc_tree_selection_selected_foreach
+  GNUNET_GTK_tree_selection_selected_foreach
     (gtk_tree_view_get_selection (GTK_TREE_VIEW (list)), &removeRow, NULL);
 }
 
 #define THUMBSIZE 128
 
 
-struct ECRS_MetaData *
+struct GNUNET_MetaData *
 getMetaDataFromList (GladeXML * xml,
                      const char *name, const char *previewName)
 {
@@ -292,7 +298,7 @@ getMetaDataFromList (GladeXML * xml,
   GtkWidget *preview;
   GtkTreeModel *metamodel;
   GtkTreeIter iter;
-  struct ECRS_MetaData *meta;
+  struct GNUNET_MetaData *meta;
   EXTRACTOR_KeywordType type;
   GdkPixbuf *pixbuf;
   GdkPixbuf *out;
@@ -304,7 +310,7 @@ getMetaDataFromList (GladeXML * xml,
 
   metaList = glade_xml_get_widget (xml, name);
   metamodel = gtk_tree_view_get_model (GTK_TREE_VIEW (metaList));
-  meta = ECRS_createMetaData ();
+  meta = GNUNET_meta_data_create ();
   if (gtk_tree_model_get_iter_first (metamodel, &iter))
     {
       do
@@ -312,8 +318,8 @@ getMetaDataFromList (GladeXML * xml,
           gtk_tree_model_get (metamodel,
                               &iter,
                               META_TYPE, &type, META_VALUE, &mvalue, -1);
-          ECRS_addToMetaData (meta, type, mvalue);
-          FREENONNULL (mvalue);
+          GNUNET_meta_data_insert (meta, type, mvalue);
+          GNUNET_free_non_null (mvalue);
         }
       while (gtk_tree_model_iter_next (metamodel, &iter));
     }
@@ -357,33 +363,34 @@ getMetaDataFromList (GladeXML * xml,
               binary = EXTRACTOR_binaryEncode ((const unsigned char *) thumb,
                                                length);
               free (thumb);
-              ECRS_addToMetaData (meta, EXTRACTOR_THUMBNAIL_DATA, binary);
-              FREE (binary);
+              GNUNET_meta_data_insert (meta, EXTRACTOR_THUMBNAIL_DATA,
+                                       binary);
+              GNUNET_free (binary);
             }
         }
     }
   return meta;
 }
 
-struct ECRS_URI *
+struct GNUNET_ECRS_URI *
 getKeywordURIFromList (GladeXML * xml, const char *name)
 {
   GtkTreeModel *keymodel;
   GtkTreeView *keyList;
   GtkTreeIter iter;
-  struct ECRS_URI *keywordURI;
+  struct GNUNET_ECRS_URI *keywordURI;
   char *mvalue;
   char **keywords;
   unsigned int kpos;
   unsigned int ksize;
 
-  DEBUG_BEGIN ();
+  GNUNET_GTK_DEBUG_BEGIN ();
   keyList = GTK_TREE_VIEW (glade_xml_get_widget (xml, name));
   keymodel = gtk_tree_view_get_model (keyList);
 
   keywords = NULL;
   ksize = 0;
-  GROW (keywords, ksize, 64);
+  GNUNET_array_grow (keywords, ksize, 64);
   kpos = 0;
   if (gtk_tree_model_get_iter_first (keymodel, &iter))
     {
@@ -392,53 +399,20 @@ getKeywordURIFromList (GladeXML * xml, const char *name)
           gtk_tree_model_get (keymodel, &iter, 0, &mvalue, -1);
           keywords[kpos++] = mvalue;
           if (kpos == ksize)
-            GROW (keywords, ksize, kpos * 2);
+            GNUNET_array_grow (keywords, ksize, kpos * 2);
         }
       while (gtk_tree_model_iter_next (keymodel, &iter));
     }
-  keywords[kpos] = NULL;
-
-  keywordURI = ECRS_keywordsToUri ((const char **) keywords);
+  keywordURI =
+    GNUNET_ECRS_keyword_command_line_to_uri (NULL, kpos,
+                                             (const char **) keywords);
   while (kpos > 0)
-    FREE (keywords[--kpos]);
-  GROW (keywords, ksize, 0);
-  DEBUG_END ();
+    GNUNET_free (keywords[--kpos]);
+  GNUNET_array_grow (keywords, ksize, 0);
+  GNUNET_GTK_DEBUG_END ();
   return keywordURI;
 }
 
-char *
-updateIntervalToString (TIME_T interval)
-{
-  if (interval == ECRS_SBLOCK_UPDATE_SPORADIC)
-    return strdup (_("--sporadic update--"));
-  if (interval == ECRS_SBLOCK_UPDATE_NONE)
-    return strdup (_("--no update--"));
-  return string_get_fancy_time_interval (interval * cronSECONDS);
-}
-
-int
-parseTimeInterval (const char *timeSpec, TIME_T * interval)
-{
-  if (timeSpec == NULL)
-    return SYSERR;
-  if (0 == strcmp (_("--sporadic update--"), timeSpec))
-    *interval = ECRS_SBLOCK_UPDATE_SPORADIC;
-  else if (0 == strcmp (_("--no update--"), timeSpec))
-    *interval = ECRS_SBLOCK_UPDATE_NONE;
-  else
-    return parseTime (timeSpec, interval);
-  return OK;
-}
-
-int
-tryParseTimeInterval (GladeXML * xml,
-                      const char *intervalComboBoxEntry, TIME_T * interval)
-{
-  return parseTimeInterval (getEntryLineValue (xml,
-                                               intervalComboBoxEntry),
-                            interval);
-}
-
 unsigned int
 getSpinButtonValue (GladeXML * xml, const char *spinButtonName)
 {
@@ -451,33 +425,33 @@ getSpinButtonValue (GladeXML * xml, const char *spinButtonName)
 
 
 char *
-getMimeTypeFromMetaData (const struct ECRS_MetaData *meta)
+getMimeTypeFromMetaData (const struct GNUNET_MetaData *meta)
 {
   char *mime;
 
-  mime = ECRS_getFromMetaData (meta, EXTRACTOR_MIMETYPE);
+  mime = GNUNET_meta_data_get_by_type (meta, EXTRACTOR_MIMETYPE);
   if (mime == NULL)
-    mime = STRDUP (_("unknown"));
-  return validate_utf8 (mime);
+    mime = GNUNET_strdup (_("unknown"));
+  return GNUNET_GTK_validate_utf8 (mime);
 }
 
 char *
-getFileNameFromMetaData (const struct ECRS_MetaData *meta)
+getFileNameFromMetaData (const struct GNUNET_MetaData *meta)
 {
   char *name;
 
-  name = ECRS_getFirstFromMetaData (meta,
-                                    EXTRACTOR_FILENAME,
-                                    EXTRACTOR_TITLE,
-                                    EXTRACTOR_ARTIST,
-                                    EXTRACTOR_AUTHOR,
-                                    EXTRACTOR_PUBLISHER,
-                                    EXTRACTOR_CREATOR,
-                                    EXTRACTOR_PRODUCER,
-                                    EXTRACTOR_UNKNOWN, -1);
+  name = GNUNET_meta_data_get_first_by_types (meta,
+                                              EXTRACTOR_FILENAME,
+                                              EXTRACTOR_TITLE,
+                                              EXTRACTOR_ARTIST,
+                                              EXTRACTOR_AUTHOR,
+                                              EXTRACTOR_PUBLISHER,
+                                              EXTRACTOR_CREATOR,
+                                              EXTRACTOR_PRODUCER,
+                                              EXTRACTOR_UNKNOWN, -1);
   if (name == NULL)
     {
-      name = STRDUP (_("no name given"));
+      name = GNUNET_strdup (_("no name given"));
     }
   else
     {
@@ -486,29 +460,30 @@ getFileNameFromMetaData (const struct ECRS_MetaData *meta)
       while (NULL != (dotdot = strstr (name, "..")))
         dotdot[0] = dotdot[1] = '_';
     }
-  return validate_utf8 (name);
+  return GNUNET_GTK_validate_utf8 (name);
 }
 
 char *
-getDescriptionFromMetaData (const struct ECRS_MetaData *meta)
+getDescriptionFromMetaData (const struct GNUNET_MetaData *meta)
 {
   char *desc;
 
-  desc = ECRS_getFirstFromMetaData (meta,
-                                    EXTRACTOR_DESCRIPTION,
-                                    EXTRACTOR_GENRE,
-                                    EXTRACTOR_ALBUM,
-                                    EXTRACTOR_COMMENT,
-                                    EXTRACTOR_SUBJECT,
-                                    EXTRACTOR_FORMAT,
-                                    EXTRACTOR_SIZE, EXTRACTOR_KEYWORDS, -1);
+  desc = GNUNET_meta_data_get_first_by_types (meta,
+                                              EXTRACTOR_DESCRIPTION,
+                                              EXTRACTOR_GENRE,
+                                              EXTRACTOR_ALBUM,
+                                              EXTRACTOR_COMMENT,
+                                              EXTRACTOR_SUBJECT,
+                                              EXTRACTOR_FORMAT,
+                                              EXTRACTOR_SIZE,
+                                              EXTRACTOR_KEYWORDS, -1);
   if (desc == NULL)
-    desc = STRDUP ("");
-  return validate_utf8 (desc);
+    desc = GNUNET_strdup ("");
+  return GNUNET_GTK_validate_utf8 (desc);
 }
 
 GdkPixbuf *
-getThumbnailFromMetaData (const struct ECRS_MetaData * meta)
+getThumbnailFromMetaData (const struct GNUNET_MetaData * meta)
 {
   GdkPixbuf *pixbuf;
   GdkPixbufLoader *loader;
@@ -516,7 +491,7 @@ getThumbnailFromMetaData (const struct ECRS_MetaData * meta)
   unsigned char *thumb;
 
   thumb = NULL;
-  ts = ECRS_getThumbnailFromMetaData (meta, &thumb);
+  ts = GNUNET_meta_data_get_thumbnail (meta, &thumb);
   if (ts != 0)
     {
       loader = gdk_pixbuf_loader_new ();
@@ -531,24 +506,10 @@ getThumbnailFromMetaData (const struct ECRS_MetaData * meta)
     {
       pixbuf = NULL;
     }
-  FREENONNULL (thumb);
+  GNUNET_free_non_null (thumb);
   return pixbuf;
 }
 
-GtkWidget *
-extractMainWidgetFromWindow (GladeXML * xml, const char *windowName)
-{
-  GtkContainer *window;
-  GtkWidget *ret;
-
-  window = GTK_CONTAINER (glade_xml_get_widget (xml, windowName));
-  ret = gtk_bin_get_child (GTK_BIN (window));
-  gtk_widget_ref (ret);
-  gtk_container_remove (window, ret);
-  gtk_widget_destroy (GTK_WIDGET (window));
-  return ret;
-}
-
 int
 getToggleButtonValue (GladeXML * xml, const char *widgetName)
 {
@@ -568,4 +529,24 @@ getEntryLineValue (GladeXML * xml, const char *widgetName)
   return gtk_entry_get_text (entry);
 }
 
+
+void 
+open_meta_data_display_dialog(const struct GNUNET_MetaData * meta,
+			      const char * title)
+{
+  GladeXML *xml;
+  GtkWidget *dialog;
+
+  xml = glade_xml_new (GNUNET_GTK_get_glade_filename (),
+                       "metaDisplayDialog", PACKAGE_NAME);
+  GNUNET_GTK_connect_glade_with_plugins (xml);
+  dialog = glade_xml_get_widget (xml, "metaDisplayDialog");
+  gtk_window_set_title (GTK_WINDOW (dialog), title);
+  createMetaDataListTreeView (xml, "metaDisplayDialogTreeView", NULL, meta);
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+  UNREF (xml);
+}
+
+
 /* end of meta.c */
